package Langertha::Spec::Ollama;
# ABSTRACT: Pre-computed OpenAPI operations for Ollama
our $VERSION = '0.404';

# AUTO-GENERATED by maint/generate_spec_data.pl -- do not edit manually.
# Source: share/ollama.yaml (12 operations)

my $DATA;

sub data {
  $DATA //= {
    server_url => 'http://localhost:11434',
    operations => {
      'chat' => { method => 'POST', path => '/api/chat', content_type => 'application/json' },
      'copy' => { method => 'POST', path => '/api/copy', content_type => 'application/json' },
      'create' => { method => 'POST', path => '/api/create', content_type => 'application/json' },
      'delete' => { method => 'DELETE', path => '/api/delete', content_type => 'application/json' },
      'embed' => { method => 'POST', path => '/api/embed', content_type => 'application/json' },
      'generate' => { method => 'POST', path => '/api/generate', content_type => 'application/json' },
      'list' => { method => 'GET', path => '/api/tags' },
      'ps' => { method => 'GET', path => '/api/ps' },
      'pull' => { method => 'POST', path => '/api/pull', content_type => 'application/json' },
      'push' => { method => 'POST', path => '/api/push', content_type => 'application/json' },
      'show' => { method => 'POST', path => '/api/show', content_type => 'application/json' },
      'version' => { method => 'GET', path => '/api/version' },
    },
  };
  return $DATA;
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Langertha::Spec::Ollama - Pre-computed OpenAPI operations for Ollama

=head1 VERSION

version 0.404

=head1 SUPPORT

=head2 Issues

Please report bugs and feature requests on GitHub at
L<https://github.com/Getty/langertha/issues>.

=head2 IRC

Join C<#langertha> on C<irc.perl.org> or message Getty directly.

=head1 CONTRIBUTING

Contributions are welcome! Please fork the repository and submit a pull request.

=head1 AUTHOR

Torsten Raudssus <getty@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2026 by Torsten Raudssus L<https://raudssus.de/>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
